<?php
// Obtaining information from Hashcat

require_once("inc/guard.inc.php");
require_once("inc/config.inc.php"); // For $config['project_user']
require_once("inc/system.inc.php");

function get_string_between($string, $start, $end){
    $string = ' ' . $string;
    $ini = strpos($string, $start);
    if ($ini == 0) return '';
    $ini += strlen($start);
    $len = strpos($string, $end, $ini) - $ini;
    return substr($string, $ini, $len);
}

function get_info_table($header, $text) {
    $len_all = $text;    
}

function get_hashtypes_by_categories() {
    global $config;
    
    $categories = array();
    
    $cmd = $config['hashcat_bin_fullpath'] . " --help";
    $result = do_as_boincuser($cmd);
    $header = "======+==================================================+======================================\n";
    $info = get_string_between($result, $header, "\n\n");
    
    $formats = array();
    
    
    foreach(preg_split("/((\r?\n)|(\r\n?))/", $info) as $line){
        $data = explode('|', $line);
        
        $output_hashid = trim($data[0]);
        
        if (is_numeric($output_hashid)) {
            if (!isset($categories[trim($data[2])])) {
                $categories[trim($data[2])] = array();
            }
            array_push($categories[trim($data[2])],
                array(
                    'id' => intval($output_hashid),
                    'name' => trim($data[1])
                )
            );
        } else {
            continue;
        }
    } 
    
    // Manually define TrueCrypt options
    
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 6211, 'name' => "TrueCrypt: RIPEMD-160, XTS 512b pure AES/Serpent/Twofish"));
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 6212, 'name' => "TrueCrypt: RIPEMD-160, XTS 1024b A/S/2F/A-2F/S-A/2F-S"));
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 6213, 'name' => "TrueCrypt: RIPEMD-160, XTS 1536b all"));
    
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 6221, 'name' => "TrueCrypt: SHA-512, XTS 512b pure AES/Serpent/Twofish"));
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 6222, 'name' => "TrueCrypt: SHA-512, XTS 1024b A/S/2F/A-2F/S-A/2F-S"));
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 6223, 'name' => "TrueCrypt: SHA-512, XTS 1536b all"));
    
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 6231, 'name' => "TrueCrypt: Whirlpool, XTS 512b pure AES/Serpent/Twofish"));
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 6232, 'name' => "TrueCrypt: Whirlpool, XTS 1024b A/S/2F/A-2F/S-A/2F-S"));
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 6233, 'name' => "TrueCrypt: Whirlpool, XTS 1536b all"));
    
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 6241, 'name' => "TrueCrypt: RIPEMD-160 bootm, XTS 512b pure AES/Serpent/Twofish"));
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 6242, 'name' => "TrueCrypt: RIPEMD-160 bootm, XTS 1024b A/S/2F/A-2F/S-A/2F-S"));
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 6243, 'name' => "TrueCrypt: RIPEMD-160 bootm, XTS 1536b all"));
    
    
    // Manually define VeraCrypt options
    
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 13711, 'name' => "VeraCrypt: RIPEMD-160, XTS 512b pure AES/Serpent/Twofish"));
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 13712, 'name' => "VeraCrypt: RIPEMD-160, XTS 1024b A/S/2F/A-2F/S-A/2F-S"));
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 13713, 'name' => "VeraCrypt: RIPEMD-160, XTS 1536b all"));
    
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 13721, 'name' => "VeraCrypt: SHA-512, XTS 512b pure AES/Serpent/Twofish"));
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 13722, 'name' => "VeraCrypt: SHA-512, XTS 1024b A/S/2F/A-2F/S-A/2F-S"));
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 13723, 'name' => "VeraCrypt: SHA-512, XTS 1536b all"));
    
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 13731, 'name' => "VeraCrypt: Whirlpool, XTS 512b pure AES/Serpent/Twofish"));
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 13732, 'name' => "VeraCrypt: Whirlpool, XTS 1024b A/S/2F/A-2F/S-A/2F-S"));
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 13733, 'name' => "VeraCrypt: Whirlpool, XTS 1536b all"));
    
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 13741, 'name' => "VeraCrypt: RIPEMD-160 bootm, XTS 512b pure AES/Serpent/Twofish"));
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 13742, 'name' => "VeraCrypt: RIPEMD-160 bootm, XTS 1024b A/S/2F/A-2F/S-A/2F-S"));
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 13743, 'name' => "VeraCrypt: RIPEMD-160 bootm, XTS 1536b all"));
    
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 13751, 'name' => "VeraCrypt: SHA-256, XTS 512b pure AES/Serpent/Twofish"));
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 13752, 'name' => "VeraCrypt: SHA-256, XTS 1024b A/S/2F/A-2F/S-A/2F-S"));
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 13753, 'name' => "VeraCrypt: SHA-256, XTS 1536b all"));
    
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 13761, 'name' => "VeraCrypt: SHA-256 bootm, XTS 512b pure AES/Serpent/Twofish"));
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 13762, 'name' => "VeraCrypt: SHA-256 bootm, XTS 1024b A/S/2F/A-2F/S-A/2F-S"));
    array_push($categories["Full-Disk Encryption (FDE)"], array('id' => 13763, 'name' => "VeraCrypt: SHA-256 bootm, XTS 1536b all"));
    
    return $categories;
}

function get_hashtypes() {
    $categories = get_hashtypes_by_categories();
    $hashtypes = array();
    
    foreach ($categories as $cat => $types) {
        foreach ($types as $type) {
            array_push($hashtypes, $type);
        }
    }
    
    return $hashtypes;
}

// Returns an associative array hash_type(ID) => hash type name
function get_hashtypes_idtoname() {
    $hashtypes = get_hashtypes();
    $id_to_name = array();
    
    foreach ($hashtypes as $hashtype) {
        $id_to_name[$hashtype['id']] = $hashtype['name'];
    }
    
    return $id_to_name;
}

function hash_type_exists($hash_type) {
    /*
    $types = get_hashtypes();
    $exists = false;
    
    foreach ($types as $type) {
        if ($hash_type == $type['id']) {
            $exists = true;
            break;
        }
    }
    */
    
    return array_key_exists($hash_type, get_hashtypes_idtoname());
}

// Calls hashcat to parse the hash
// REQUIRES: OpenCL support for CPU (e.g. Intel OpenCL)
// MAY TAKE SOME TIME
function verify_hash_format($hash_type, $hash) {
    $ok = true;
    global $config;
    if (!is_numeric($hash_type)) {
        return false;
    }
    
    // Prepare hash file
    $hashfile = "/tmp/h" . date('YmdHis') . $hash_type;
    $cmd = "echo -n '" . addslashes($hash) . "' > $hashfile && chmod 775 $hashfile";
    shell_exec($cmd);
    
    // Verify hash
    $args = "-m $hash_type $hashfile -a 3 ?a?a?a -l 1 -w 1 --potfile-disable --logfile-disable";
    $cmd = "sh -c '";
    $cmd .= "cd " . $config['hashcat_bin_dir'];
    $cmd .= " && " .  $config['hashcat_bin_fullpath'] . " $args 2>&1";
    $cmd .= "'";
    $result = do_as_boincuser($cmd);
    
    // Delete hash file
    $cmd = "rm $hashfile";
    shell_exec($cmd);
    
    // Check results
    if (strpos($result, 'exception') !== false) {
        return false;
    }
            
    return true;
}

function hc_keyspace_from_mask($mask) {
    global $config;
    $cmd = "sh -c '";
    $cmd .= "cd " . $config['hashcat-utils_bin_dir'];
    $cmd .= " && ";
    $cmd .= $config['keyspace_bin_fullpath'] . " $mask 2>&1";
    $cmd .= "'";
    $result = do_as_boincuser($cmd);
    return intval($result);
}

function hc_keyspace_from_mask_hashtype($mask, $hashtype) {
    global $config;
    $cmd = "sh -c '";
    $cmd .= "cd " . $config['hashcat_bin_dir'];
    $cmd .= " && ";
    $cmd .= $config['hashcat_bin_fullpath'] . " -m $hashtype --keyspace -a 3 $mask 2>&1";
    $cmd .= "'";
    $result = do_as_boincuser($cmd);
    return intval($result);
}

function hc_keyspace_dict($path) {
    global $config;
    $cmd = "sh -c '";
    $cmd .= "cd " . $config['hashcat_bin_dir'];
    $cmd .= " && ";
    $cmd .= $config['hashcat_bin_fullpath'] . " --keyspace -a 0 $path 2>&1";
    $cmd .= "'";
    $result = do_as_boincuser($cmd);
    return intval($result);
}





 /*
    global $config;
    $cmd = $config['hashcat_bin_fullpath'] . " --help";
    $result = do_as_boincuser($cmd);
    $header = "======+==================================================+======================================\n";
    $info = get_string_between($result, $header, "\n\n");
    
    $formats = array();
    foreach(preg_split("/((\r?\n)|(\r\n?))/", $info) as $line){
        $data = explode('|', $line);
        
        $output_hashid = trim($data[0]);
        
        if (is_numeric($output_hashid)) {
            $format['id'] = intval($output_hashid);
        } else {
            continue;
        }
        
        
        $format['name'] = trim($data[1]);
        $format['category'] = trim($data[2]);
        array_push($formats, $format);
    } 
    * 
    * 
    * 
    * function get_hashtypes_idtoname() {
    global $config;
    $cmd = $config['hashcat_bin_fullpath'] . " --help";
    $result = do_as_boincuser($cmd);
    $header = "======+==================================================+======================================\n";
    $info = get_string_between($result, $header, "\n\n");
    
    $formats = array();
    foreach(preg_split("/((\r?\n)|(\r\n?))/", $info) as $line){
        $data = explode('|', $line);
        $id = intval(trim($data[0]));
        $name = trim($data[1]);
        //$category = trim($data[2]);
        $formats[$id] = $name;
    } 
    return $formats;
}
    */
