<?php
// Checks if the syntax of the mask is ok
function check_mask_syntax($mask) {
    $match = array();
    if (preg_match_all('/^(\?[ludhHsab]|[ -~])+$/', $mask, $match, PREG_SET_ORDER, 0)) {
    //if (preg_match_all('/^(\?[ludhHsab])+$/', $mask, $match, PREG_SET_ORDER, 0)) {
        return true;
    } else {
        return false;
    }
}

// Compute (correctly) the number of all possible passwords
function compute_keyspace($mask) {
    $keyspace = 1;
    $question_mark = false;
    for ($pos = 0; $pos < strlen($mask); $pos++) {
        if ($mask[$pos] == '?') {
            if ($question_mark) { // ? after ?
                continue;
            } else {
                $question_mark = true;
            }
        } else {
            if ($question_mark) {
                $question_mark = false;
                $group_symbol = $mask[$pos];
                switch($group_symbol) {
                    case 'l':
                        $keyspace *= 26;
                    break;
                    case 'u':
                        $keyspace *= 26;
                    break;
                    case 'd':
                        $keyspace *= 10;
                    break;
                    case 'h':
                        $keyspace *= 16;
                    break;
                    case 'H':
                        $keyspace *= 16;
                    break;
                    case 's':
                        $keyspace *= 33;
                    break;
                    case 'a':
                        $keyspace *= 95;
                    break;
                    case 'b':
                        $keyspace *= 256;
                    break;
                    default:
                        add_error("Unknown character in mask!");
                        return 0;
                    break;
                }
            }
        }
    }
    return $keyspace;
}
/*
function compute_keyspace($mask) {
    // Assuming the mask is in the correct format
    $clean_mask = str_replace('?', '', $mask); // Remove "?" from mask
    $keyspace = 1;
    for ($i = 0; $i < strlen($clean_mask); $i++) {
        switch(substr($clean_mask, $i, 1)) {
            case 'l':
                $keyspace *= 26;
            break;
            case 'u':
                $keyspace *= 26;
            break;
            case 'd':
                $keyspace *= 10;
            break;
            case 'h':
                $keyspace *= 16;
            break;
            case 'H':
                $keyspace *= 16;
            break;
            case 's':
                $keyspace *= 33;
            break;
            case 'a':
                $keyspace *= 95;
            break;
            case 'b':
                $keyspace *= 256;
            break;
            default:
                add_error("Unknown character in mask!");
                return 0;
            break;
        }
    }
    $positions = strlen($mask) / 2;    
    return $keyspace;
}
*/

function process_package_brute(&$pkg, &$ok) {    
    // BRUTE --------------------------------------------------------------------------------
    $max_mask = intval($_POST['max']);
    $pkg['masks'] = array();
    $mask_id = 0;
    
    foreach($_POST['mask'] as $m) {
        array_push($pkg['masks'], $m);
        if (strlen($m) == 0) {
            add_error("Line with no mask was entered!");
            continue;
        }
        if (!check_mask_syntax($m)) {
            add_error("Invalid mask: \"<b>" . htmlentities($m) . "</b>\" !");
            continue;
        }
    }
    
    $mask_table = array();
    if ($ok) {
        // Generate mask table
        
        $keyspace_total = 0;
        $hc_keyspace_total = 0;
        for ($i = 0; $i < count($pkg['masks']); $i++) {
            $mask_table[$i]['mask'] = $pkg['masks'][$i];
            $keyspace_for_mask = compute_keyspace($pkg['masks'][$i]);          // Real keyspace
            //$hc_keyspace_for_mask = hc_keyspace_from_mask($pkg['masks'][$i]);  // Hashcat keyspace
            $hc_keyspace_for_mask = hc_keyspace_from_mask_hashtype($pkg['masks'][$i], $pkg['hash_type']); // Hashcat keyspace (for a given hash type)
            $keyspace_total += $keyspace_for_mask;
            $hc_keyspace_total += $hc_keyspace_for_mask;
            $mask_table[$i]['keyspace'] = $keyspace_for_mask;
            $mask_table[$i]['hc_keyspace'] = $hc_keyspace_for_mask;
            
            if ($pkg['info_only']) {
                echo "<p>";
                echo "Mask: " . $pkg['masks'][$i] . " | real keyspace: $keyspace_for_mask | hashcat keyspace: $hc_keyspace_for_mask";
                echo "</p>\n";
            }
            
        }
        $pkg['keyspace'] = $keyspace_total;
        $pkg['hc_keyspace'] = $hc_keyspace_total;
        $pkg['mask_table'] = $mask_table;
    }
}

function postprocess_package_brute(&$pkg, &$ok, $package_id) {
    global $config;
    global $mysqli;
    
    foreach ($pkg['mask_table'] as $mask_table_row) {
        $mask = $mask_table_row['mask'];
        $keyspace = $mask_table_row['keyspace'];
        $hc_keyspace = $mask_table_row['hc_keyspace'];
        $db_data = array(
            'package_id' => "$package_id",
            'mask' => "'" . mysqli_real_escape_string($mysqli, $mask) . "'",
            'current_index' => "0",
            'keyspace' => "$keyspace",
            'hc_keyspace' => "$hc_keyspace",
        );
        
        $sql = "INSERT INTO `" . $config['mysql_prefix'] . "mask` ";
        $sql .= sql_data_part($db_data);
        
        if (!$result = $mysqli->query($sql)) {
            $ok = false;
            add_error('Postprocessing failed: Unable to add masks for a new package!');
        }
    }
}
?>
