<?php
require_once("inc/guard.inc.php");
require_once("inc/mysql.inc.php");

$found = false;
$package = NULL;

// Try to obtain package info
if (isset($_GET['id']) && is_numeric($_GET['id'])) {
    $package_id = $_GET['id'];
    $sql = "SELECT * FROM `" . $config['mysql_prefix'] . "package` WHERE id = $package_id";
    if ($result = $mysqli->query($sql)) {
        $package = $result->fetch_array(MYSQLI_ASSOC);
    } else {
        die("<p>Unable to find package $package_id!</p><br/>\n");
    }
} else {
    die("<p>No package specified!</p><br/>\n");
}

// Load necessary functions
require_once("inc/package_builder.inc.php");
require_once("inc/hashcat_info.inc.php");

$hashtypes_idtoname = get_hashtypes_idtoname();

// Display package info
echo "<h1>Package " . htmlentities($package['name']) . "</h1>";

// GENERAL INFO ===================================================
echo "<b>ID:</b> ";
echo $package_id;
echo "<br/>\n";

echo "<b>Comment:</b> ";
echo htmlentities($package['comment']);
echo "<br/>\n";

echo "<b>Attack type:</b> ";
echo $attack_caption[$package['attack']];
echo "<br/>\n";

echo "<b>Hashcat attack mode:</b> ";
echo $package['attack_mode'];
echo "<br/>\n";

echo "<b>Status:</b> ";
$status = $package['status'];
if ($status == 0) {
    echo "<b><span style=\"color: #997300;\">ready</span></b>";
} else if ($status == 1) {
    $progress = 100;
    echo "<b><span style=\"color: #000099;\">finished</span></b>";
} else if ($status == 2) {
    $progress = 100;
    echo "<b><span style=\"color: #FF0000;\">exhausted</span></b>";
} else if ($status == 3) {
    echo "<b><span style=\"color: #000000;\">malformed</span></b>";
} else if ($status == 4) {
    echo "<b><span style=\"color: #808080;\">timeout</span></b>";
} else if ($status >= 10) {
    echo "<b><span style=\"color: #006600;\">running</span></b>";
}
echo "<br/>";

echo "<b>Start time:</b> ";
echo date('j F Y h:m:s',strtotime($package['time_start']));
echo "<br/>";

echo "<b>End time:</b> ";
if (empty($package['time_end'])) {
    echo date('j F Y h:m:s',strtotime($package['time_end']));
} else {
    echo "never (unless exhausted or a password is found)";
}

// HASH INFO ===================================================
echo "<h2>Hash info</h2>\n";
echo "<b>Hash type:</b> ";
echo $hashtypes_idtoname[$package['hash_type']] . " (" . $package['hash_type'] . ")";
echo "<br/>";

echo "<b>Hash:</b> ";
echo htmlentities($package['hash']);
echo "<br/>";

echo "<b>Present in cache:</b> ";
$cached_base64_pw = hashcache_lookup($package['hash'], $package['hash_type']);
if ($cached_base64_pw == NULL) {
    echo "no";
} else {
    echo "yes (password: <span class=\"password\">" . htmlentities(base64_decode($cached_base64_pw)) . "</span>)";
}
echo "<br/>";

// ATTACK INFO ===================================================

echo "<h2>" . $attack_caption[$package['attack']] . " info</h2>\n";

if (intval($package['hc_keyspace']) > 0) {
    $normalization = intval($package['keyspace']) / intval($package['hc_keyspace']);
} else {
    $normalization = 1;
}

echo "<b>Real passwords verified:</b> ";
echo intval($package['indexes_verified']) * $normalization . " of " . $package['keyspace'];;
echo "<br/>";

echo "<b>Hashcat indexes verified:</b> ";
echo $package['indexes_verified'] . " of " . $package['hc_keyspace'];
echo "<br/>";

echo "<b>Hashcat keyspace normalization:</b> ";
echo $normalization;
echo "<br/>";


$progress = 0;
if (!empty($package['result'])) {
    $progress = 100;
} else if (intval($package['hc_keyspace']) > 0) {
    $progress = round((intval($package['indexes_verified']) / intval($package['hc_keyspace'])) * 100, 2); // TOOD - fix for 
}
echo "<b>Total progress:</b> " . number_format($progress, 2) . " %<br/>\n";

$pw = $package['result'];

echo "<b>Password found:</b> ";
if (empty($pw)) {
    echo "<font color=\"red\"><b>no</b></font>";
} else {
    echo "<font color=\"green\"><b>YES</b></font>";
    echo "<br/><b>Password:</b> <span class=\"password\">" . base64_decode($pw) . "</span><br/>";
}
echo "<br/>";


if ($package['attack'] == 'brute') {
    $masks = get_masks_for_package($package_id);
    echo "<table id=\"hashlist\" class=\"striped\" style=\"margin-top: 12px;\">\n";
    echo "<tr class=\"heading\">\n";
    echo "<td>mask</td>\n";
    echo "<td>real keyspace</td>\n";
    echo "<td>Hashcat keyspace</td>\n";
    echo "<td>current index</td>\n";
    echo "<td>progress</td>\n";
    echo "</tr>\n";
    foreach ($masks as $mask) {
        echo "<tr>\n";
        echo "<td><b>" . htmlentities($mask['mask']) . "</b></td>\n";
        echo "<td>" . $mask['keyspace'] . "</td>";
        echo "<td>" . $mask['hc_keyspace'] . "</td>";
        echo "<td>" . $mask['current_index'] . "</td>";
        $progress = 0;
        if (intval($mask['hc_keyspace']) != 0) {
            $progress = round((intval($mask['current_index']) / intval($mask['hc_keyspace'])) * 100, 2);
        }
        echo "<td>" . number_format($progress, 2) . " %</td>";
        echo "</tr>\n";
    }
    echo "</table>\n";    
} else if ($package['attack'] == 'dict') {
    echo "<b>Dictionary:</b> ";
    echo "<a href=\"javascript:examine_dict('" . htmlentities($package['dict1']) . "')\">";
    echo $package['dict1'];
    echo "</a>";
    echo "<br/>";
}

// HOSTS INFO ===================================================

echo "<h2>Host mapped to package (<a href=\"?i=host_mapping&pkg_id=$package_id\">manage</a>)</h2>";
// Get mapped hosts
$mapped = array();

$sql = "SELECT id, userid, domain_name, last_ip_addr, p_model, os_name, os_version FROM ";
$sql .= "host WHERE id";
$sql .=" IN (SELECT boinc_host_id FROM `" . $config['mysql_prefix'] . "host_activity` WHERE package_id = $package_id) ORDER BY id DESC";

if (!$result = $mysqli->query($sql)) {
    echo mysqli_error($mysqli);
    die("<p>Unable to read host information!</p><br/>\n");
}

if ($result->num_rows > 0) {
    
    echo "<table id=\"sql_result\" class=\"striped\">\n";
    echo "<tr class=\"heading\">\n";
    echo "<td>ID</td>\n";
    echo "<td>user</td>\n";
    echo "<td>name</td>\n";
    echo "<td>last IP</td>\n";
    echo "<td>CPU</td>\n";
    echo "<td>OS</td>\n";
    while ($host = $result->fetch_array(MYSQLI_ASSOC)) {
        echo "<tr>\n";
        echo "<td>\n";
        echo $host['id'];
        echo "</td>\n";
        echo "<td>\n";
        $sql2 = "SELECT name FROM `user` WHERE id = " . $host['userid'];
        if (!$result2 = $mysqli->query($sql2)) {
            echo "?";
        }
        if ($result2->num_rows > 0) {
            if ($user =  $result2->fetch_array(MYSQLI_ASSOC)) {
                echo htmlentities(stripcslashes($user['name']));
            } else {
                echo "?";
            }
        }
        echo "</td>\n";
        echo "<td>\n";
        echo htmlentities(stripslashes($host['domain_name']));
        echo "</td>\n";
        echo "<td>\n";
        echo htmlentities(stripslashes($host['last_ip_addr']));
        echo "</td>\n";
        echo "<td>\n";
        $cpu = stripslashes($host['p_model']);
        if (strlen($cpu) > 100) {
            $cpu = substr($cpu, 0, 100) . "...";
        }
        echo htmlentities($cpu);
        echo "</td>\n";
        echo "<td>\n";
        echo htmlentities(stripslashes($host['os_name'])) . " " . htmlentities(stripslashes($host['os_version']));
        echo "</td>\n";        
        echo "</tr>\n";
    }
    echo "</table>\n";
} else {
    echo "No hosts mapped to this package.<br/>";
}

// JOBS ===================================================

echo "<h2>Jobs for package</h2>";

echo "<b>Desired time for each job:</b> ";
echo $package['seconds_per_job'] . " s";
echo "<br/>\n";

$jobs = get_jobs_for_package($package_id);
echo "<table id=\"hashlist\" class=\"striped\" style=\"margin-top: 12px;\">\n";
echo "<tr class=\"heading\">\n";
echo "<td>ID</td>\n";
echo "<td>start index</td>\n";
echo "<td>Hashcat keyspace</td>\n";
echo "<td>assigned at</td>\n";
echo "<td>assigned to</td>\n";
echo "<td>cracking time</td>\n";
echo "<td>retry</td>\n";
echo "<td>finished</td>\n";
echo "</tr>\n";
foreach ($jobs as $job) {
    echo "<tr>\n";
    echo "<td>" . $job['id'] . "</td>";
    echo "<td>" . $job['start_index'] . "</td>";
    echo "<td>" . $job['hc_keyspace'] . "</td>";
    echo "<td>" . $job['time'] . "</td>";
    echo "<td>" . htmlentities(get_boinc_host_name($job['boinc_host_id'])) . "</td>";
    echo "<td>" . $job['cracking_time'] . " s</td>";
    echo "<td>";
    if (intval($job['retry']) == 1) {
        echo "yes";
    } else {
        echo "no";
    }
    echo "</td>";
    echo "<td>";
    if (intval($job['finished']) == 1) {
        echo "<font color=\"green\">yes</font>";
    } else {
        echo "<font color=\"red\">no</font>";
    }
    echo "</td>";
    echo "</tr>\n";
}
echo "</table>\n";  
?>


